#!/usr/bin/python
# -*- coding:utf-8 -*-

import logging
import sys
import time
import math
import smbus

PCF85063_ADDRESS    =      (0x51)


CONTROL_1_REG      = 0x00  
CONTROL_2_REG       = 0x01 
OFFSET_REG          = 0x02 
RAM_BYTE_REG        = 0x03 
SECONDS_REG         = 0x04 
MINUTES_REG         = 0x05 
HOURS_REG           = 0x06 
DAYS_REG            = 0x07 
WEEKDAYS_REG        = 0x08 
MONTHS_REG          = 0x09 
YEARS_REG           = 0x0A 
SECOND_ALARM_REG    = 0x0B 
MINUTES_ALARM_REG   = 0x0C 
HOUR_ARARM_REG      = 0x0D  
DAY_ALARM_REG       = 0x0E
WEEKDAY_ALARM_REG   = 0x0F
TIMER_VALUE_REG     = 0x10
TIMER_MODE_REG      = 0x11


def decToBcd( val):
  return ( (int(val/10)*16) + (val%10) );


def bcdToDec( val):
    return ( (int(val/16)*10) + (val%16) );


class PCF85063:
    def __init__(self, address=PCF85063_ADDRESS):
        
        self.i2c = smbus.SMBus(10)
        self.address = address
        # self.Write_Byte(CONTROL_1_REG,0x00)
        inspect = 0
        self.REST()
        time.sleep(0.5)
        self.Write_Byte(SECONDS_REG,self.Read_Byte(SECONDS_REG)|0x80)
        self.Write_Byte(CONTROL_2_REG,0x80)
        while(1):
            self.Write_Byte(SECONDS_REG,self.Read_Byte(SECONDS_REG)&0x7F)
            if((self.Read_Byte(SECONDS_REG)&0x80) == 0):
                break
            time.sleep(0.5)
            inspect  = inspect+1 
            if(inspect>5):
                print ("Clock stability unknown")
                break
        # self.Write_Byte(TIMER_MODE_REG,0x00)
        
    def Read_Byte(self, Addr):
        
        Addr = ( Addr) & 0xFF
        while(1):
            try:
                x  = self.i2c.read_byte_data(self.address, Addr)
                if(x != None):
                    return x
            except :
                pass



    def Read_Word(self, Addr):
        Addr = ( Addr) & 0xFF
        while(1):
            try:
                x = self.i2c.read_word_data(self.address, Addr)
                if(x != None):
                    return x
            except :
                pass


    def Write_Byte(self, Addr, val):
        Addr = ( Addr) & 0xFF
        while(1):
            try:
                self.i2c.write_byte_data(self.address, Addr, val & 0xFF)
                i = 0
                return
            except :
                pass


    def setTime_YMD(self,Years, Months, Days) :
        if(Years>99):
            Years = 99
        if(Months>12):
            Months = 12
        if(Days>31):
            Days = 31
        self.Write_Byte(YEARS_REG  ,decToBcd(Years))
        self.Write_Byte(MONTHS_REG ,decToBcd(Months)&0x1F)
        self.Write_Byte(DAYS_REG   ,decToBcd(Days)&0x3F)
        
    def setTime_HMS(self, hour,  minute,  second) :
        if(hour>23):
            hour = 23
        if(minute>59):
            minute = 59
        if(second>59):
            second = 59
        self.Write_Byte(HOURS_REG  ,decToBcd(hour)&0x3F)
        self.Write_Byte(MINUTES_REG,decToBcd(minute)&0x7F)
        self.Write_Byte(SECONDS_REG,decToBcd(second)&0x7F)
        
    def getTime(self) :
        data = [0,0,0,0,0,0]
        data[0] = bcdToDec(self.Read_Byte(YEARS_REG))
        data[1] = bcdToDec(self.Read_Byte(MONTHS_REG)&0x1F)
        data[2] = bcdToDec(self.Read_Byte(DAYS_REG)&0x3F)
        data[3] = bcdToDec(self.Read_Byte(HOURS_REG)&0x3F)
        data[4] = bcdToDec(self.Read_Byte(MINUTES_REG)&0x7F)
        data[5] = bcdToDec(self.Read_Byte(SECONDS_REG)&0x7F)
        return data
    
    
    
    def ste_alarm_Time(self, hour,  minute,  second):
        if(hour>23):
            hour = 23
        if(minute>59):
            minute = 59
        if(second>59):
            second = 59
        print ("%d  %d  %d"%(hour,minute,second))
        self.Write_Byte(HOUR_ARARM_REG   ,decToBcd(hour)&0x3F)
        self.Write_Byte(MINUTES_ALARM_REG,decToBcd(minute)&0x7F)
        self.Write_Byte(SECOND_ALARM_REG ,decToBcd(second)&0x7F)
        
        
        
    def ste_alarm_Time_Enabled(self):
        #self.Write_Byte(CONTROL_2_REG   ,self.Read_Byte(CONTROL_2_REG)|0x80) #Alarm on
        
        self.Write_Byte(HOUR_ARARM_REG   ,self.Read_Byte(HOUR_ARARM_REG)&0x7F)
        self.Write_Byte(MINUTES_ALARM_REG,self.Read_Byte(MINUTES_ALARM_REG)&0x7F)
        self.Write_Byte(SECOND_ALARM_REG ,self.Read_Byte(SECOND_ALARM_REG)&0x7F)
        
        
    
    def ste_alarm_Time_Disable(self):
        self.Write_Byte(HOUR_ARARM_REG   ,self.Read_Byte(HOUR_ARARM_REG)|0x80)
        self.Write_Byte(MINUTES_ALARM_REG,self.Read_Byte(MINUTES_ALARM_REG)|0x80)
        self.Write_Byte(SECOND_ALARM_REG ,self.Read_Byte(SECOND_ALARM_REG)|0x80)
        
        # self.Write_Byte(CONTROL_2_REG   ,self.Read_Byte(CONTROL_2_REG)&0x7F) #Alarm OFF
    
    def REST(self):
        self.Write_Byte(CONTROL_1_REG,0x58)
        

    def get_alarm_flag(self):
        #print ("0x%x"%self.Read_Byte(CONTROL_2_REG))
        if(self.Read_Byte(CONTROL_2_REG)&0x40 == 0x40):
            return 1
        else :
            return 0
        
    def clear_alarm_flag(self):
        self.Write_Byte(CONTROL_2_REG   ,self.Read_Byte(CONTROL_2_REG)&0xBF) 
        
    

    


    
    
    
    
    
    
    
        